%   function L = laplacian(img, sigma)
%
% sigma is the standard deviation of the Gaussian function used for
% differentiation; if left unspecified, sigma defaults to 1.
% Color images are converted to black and white before the Laplacian is
% computed (if you want the Laplacian of each band, call this function three
% times).

function L = laplacian(img, sigma)

% Assign a default standard deviation if needed
if nargin < 2
    sigma = 1;
end

% Make the tails of the Gaussian long enough to make truncation
% unnoticeable
tail = ceil(2.5 * sigma);
x = -tail:tail;
x = x(:);

% A one-dimensional Gaussian and its derivative
g = gauss(x, sigma);
d = gaussSecondDeriv(x, sigma);

% Convert to doubles if needed
img = double(img);

% Color-to-bw conversion, if needed
if ndims(img) == 3
    img = squeeze(img(:, :, 1) + img(:, :, 2) + img(:, :, 3)) / 3;
end

% Do the two separable convolutions, and add them
% L = conv2(d, g, img, 'valid') + conv2(g, d, img, 'valid');
% 
% % Handle image boundaries appropriately. Cannot use 'same' option
% % in conv2, because we don't want spurious edges
% [rows, cols] = size(img);
% L = adjustSize(L, rows, cols);

L = conv2(d, g, img, 'same') + conv2(g, d, img, 'same');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% With normalized == 0, returns saamples of a 1D Gaussian with the given
% standard deviation. Adjusts normalization
% so that the sum of the samples is one.
function g = gauss(x, sigma)

g = exp(- ((x / sigma) .^ 2) / 2) / sqrt(2 * pi) / sigma;
g = g / sum(g);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% With normalized == 0, returns saamples of the second derivative of a 1D Gaussian
% with the given standard deviation. Adjusts
% normalization so that the inner product of the samples with a
% unit-curvatur parabola centered at the origin is minus one.
function d2 = gaussSecondDeriv(x, sigma)

d2 = ((x/sigma) .^ 2 - 1) .* gauss(x, sigma);
d2 = d2 / sum(d2 .* (x/2) .^ 2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pads a with zeros symmetrically to obtain an image of size [rows, cols]
% (assumed to be no smaller than size(a)).

function b = adjustSize(a, rows, cols)

[aRows, aCols] = size(a);

rs = 1 + floor((rows - aRows) / 2);
re = rs + aRows - 1;

cs = 1 + floor((cols - aCols) / 2);
ce = cs + aCols - 1;

b = zeros(rows, cols);
b(rs:re, cs:ce) = a;

